# =========================================================================
# Copyright (c) 2004 Josh Glover <jmglov@jmglov.net>
#
#   This file is distributed under the terms of the BSD-2 License.
#   See the 'COPYING' file, which should have been distributed with
#   this file, for details. If you did not receive the 'COPYING' file,
#   see:
#
#   http://www.jmglov.net/opensource/licenses/bsd.txt
#
# osdt.sh
#
# DESCRIPTION:
#
#   Standard functions for all OSDT shell scripts.
#
# USAGE:
#
#   INCDIR="/usr/include"
#   source "${INCDIR}/osdt.sh"
#
# EXAMPLES:
#
#   host=osdtGetHostname
#   host=osdtGetFqdn
#   ips=osdtGetIpAddresses
#
# DEPENDENCIES:
#
#   awk (tested with GNU awk)
#   Bourne-compatible shell
#   cut (tested with cut from GNU coreutils)
#   hostname (tested with Linux net-tools, OpenBSD, and Solaris)
#   uname
#
# TODO:
#
#   - How does one determine the bloody hostname on SysV (e.g. UnixWare)?
#   - Test osdtGetGqdn() on NIS-enabled Solaris boxen where the NIS domain
#     differs from the DNS one
#   - Parse the output of /sbin/ifconfig on other Unices (especially *BSD, OSX)
#   - Nothing, this code is perfect
#
# MODIFICATIONS:
#
#   Josh Glover <jmglov@jmglov.net> (2004/06/06): Initial revision
# =========================================================================


# Functions
# =========================================================================


# Function: osdtGetFqdn()
#
# Determines the fully qualified domain name (FQDN; e.g. foo.bar.net) of the
# local host.
#
# Returns:
#
#   FQDN as a string

osdtGetFqdn() {

  # Figure out on which OS we are running
  case `uname` in

    Linux)
      return `hostname --fqdn`
      ;;

    OpenBSD)
      return `hostname`
      ;;

    ### This might return the NIS domain name instead, but I don't know ###
    ### any other way                                                   ###
    SunOS)
      return `hostname`'.'`domainname`
      ;;

    ### I do not know how to get the domain name on SysV ###
    # hostname(1) seems to be in /usr/ucb (which may or may not be in the path)
    UNIX_SV)
      return `PATH=${PATH}:/usr/ucb hostname`
      ;;

    # Unsupported OS, the best we can do is return hostname as is
    *)
      return `hostname`
      ;;

  esac # case (on which OS are we running?)

} # osdtGetFqdn()


# Function: osdtGetHostname()
#
# Determines the hostname of the local host; see also <osdtGetFqdn()>.
#
# Parameters:
#
#   fqdn - [Optional] FQDN (see <osdtGetFqdn()>); if set, just the hostname
#          portion will be returned
#
# Returns:
#
#   Hostname as a string

osdtGetHostname() {

  local fqdn="${1}"; shift

  # Grab the FQDN unless the user passed it in as a param
  if [ -z "${fqdn}" ]; then fqdn=osdtGetFqdn; fi

  return `echo ${fqdn} | cut -d'.' -f 1 2>/dev/null`

} # osdtGetHostname()


# Function: osdtGetLocalIps()
#
# Determines the IP address(es) of the local host
#
# Returns:
#
#   IP address(es) of the local host, as a list of words

osdtGetLocalIps() {

  # Grab the output of ifconfig(8), appending all the likely locations to our
  # ${PATH}, just in case
  local ifc=``

  # As far as I can see, there appear to be only two output formats for the
  # IP address of an interface from ifconfig(8): "inet addr:192.168.1.51"
  # (on my Linux box, running the net-tools version of ifconfig(8)), and
  # "inet 192.168.1.51" (on everything else).
  return `PATH=${PATH}/sbin:/usr/sbin:/usr/local/sbin:/opt/sbin ifconfig -a 2>/dev/null | grep 'inet ' 2>/dev/null | sed -e 's/^inet addr:\([0-9\.]\+\)$/\1/'`

} # osdtGetLocalIps()


# Function: osdtIsLocalhost()
#
# Determines if the specified hostname refers to the local host.
#
# Parameters:
#
#   host - hostname, FQDN, or IP address; see <osdtGetFqdn()>
#
# Returns:
#
#   0 if the hostname does refer to the local host, 1 otherwise

osdtIsLocalhost() {

  local host="${1}"; shift
  local localhost=""

  # If we did not get a host param, return failure
  if [ -z "${host}" ]; then return 1; fi

  # First of all, do we have an IP address?
  if echo "${host}" | grep '^[0-9\.]+$' >/dev/null 2>&1; then

    localhost=osdtGetLocalIp

  fi # if (we have an IP address)

  # If the host param contains a '.', we need to check it against the FQDN
  if echo "${host}" | grep '\.' >/dev/null 2>&1; then

    localhost=osdtGetFqdn

  else

    localhost=osdtGetHostname

  fi # if (do we need hostname or FQDN?)

  # If the hostnames / FQDNs match, return success
  if [ "${host}" = "${localhost}" ]; then

    return 0

  else

    return 1

  fi # if (do they match?)

} # osdtIsLocalhost()


# =========================================================================


# Main program
# =========================================================================


cmd=""
src=""
tgt=""

# Parse args
cmd="${1}"; shift
src="${1}"; shift
tgt="${1}"; shift

# Check usage
if [ -z "${cmd}" -o -z "${cmd}" -o -z "${cmd}" ]; then usage; fi

# Check ${src} for login strings (note that it is not quoted, because it might
# be multiple words, and each should be checked)
fixed_src=""
for i in ${src}; do

  # Split the word on ':'
  login=`echo ${i} | cut -d':' -f 1 -s 2>/dev/null` # -s: only if ':' is present
  file=`echo ${i}  | cut -d':' -f 2    2>/dev/null` # even if ':' is absent

  # If we have a login string, we must check the hostname
  if [ -n "${login}" ]; then

    user=`echo ${login} | cut -d'@' -f 1 -s 2>/dev/null` # only if '@' is present
    host=`echo ${login} | cut -d'@' -f 2    2>/dev/null` # even if '@' is absent

    # Check if the host is the local machine
    localhost=`hostname 2>/dev/null | cut -d'.' -f 1 2>/dev/null`
    ifc=`/sbin/ifconfig -a 2>/dev/null | grep inet 2>/dev/null`
    ips=""
    if echo "${ifc}" | grep 'inet addr:' >/dev/null 2>&1; then

      ips=`echo ${ifc} | cut -d':' -f 2 2>/dev/null | awk '{print $1}' 2>/dev/null`
    else

      awk '{print $2}'

    fi # if (Linux or Solaris?)

    for i in "${localhost}" 

  fi # if (login string present)

done # for (checking for hostnames)

# Success!
exit 0


# =========================================================================
